#version 300 es

precision highp float;
precision highp int;

in vec3 v_pos;
in vec4 v_color;

uniform mat4 u_modelViewMat;
uniform vec4 u_fogColor;
uniform float u_fogStart;
uniform float u_fogEnd;

out vec4 fragColor;

vec4 linear_fog(vec4 inColor, float vertexDistance, float fogStart, float fogEnd, vec4 fogColor) {
    float fogFactor = clamp((fogEnd - vertexDistance) / (fogEnd - fogStart), 0.0, 1.0);
    return mix(fogColor, inColor, fogFactor);
}

float linear_fog_fade(float vertexDistance, float fogStart, float fogEnd) {
    return clamp((fogEnd - vertexDistance) / (fogEnd - fogStart), 0.0, 1.0);
}

float fog_distance(mat4 modelViewMat, vec3 pos, int shape) {
    vec4 transformedPos = modelViewMat * vec4(pos, 1.0);
    if (shape == 0) {
        return length(transformedPos.xyz);
    } else {
        float distXZ = length(vec2(transformedPos.x, transformedPos.z));
        float distY = abs(transformedPos.y);
        return max(distXZ, distY);
    }
}

void main() {
    float distance = fog_distance(u_modelViewMat, v_pos, 0); // Assuming shape is 0, update if necessary
    vec4 color = linear_fog(v_color, distance, u_fogStart, u_fogEnd, u_fogColor);
    fragColor = color;
}
